%A Fast Numerical Procedure for Steady Capillary Flow in Open Channels
%J. Klatte, D. Haake, M. M. Weislogel, M. Dreyer
%Center of Applied Space Technology and Microgravity, 
%University of Bremen, Bremen, 28359
%Department of Mechanical Engineering,
%Portland State University, Portland, OR 97207
%Acta Mechanica, 2008

function NewPressure()
%This function calculates the pressure changes according to the interface 
%position for the wedge channel. In detail the function solves (eq.1) 
%and calculates the interpolation coefficients a (eq. 7). 

%-----Load variables and surface points -----------------------------------
%-----from Evolver output file SurfaceVerticesEvolver.txt------------------
file=fopen('SurfaceVerticesEvolver.txt');
    %Load parameters
  theta=fscanf(file,'%f'); fseek(file,5,0);         %contact angle wall
  Q=fscanf(file,'%f'); fseek(file,2,0);             %flow rate Q
  viscosity=fscanf(file,'%f'); fseek(file,3,0);     %dynamic viscosity eta
  density=fscanf(file,'%f'); fseek(file,3,0);       %liquid density rho
  height=fscanf(file,'%f'); fseek(file,2,0);        %channel height H
  alpha=fscanf(file,'%f'); fseek(file,5,0);         %opening angle alpha
    %Load interface points and sort them
  h=fscanf(file,'%f %f',[2 inf]);   %Interface points at y=0 in x-direction
  [h(1,:),I]=sort(h(1,:));
  h(2,:)=h(2,I);
fclose(file);
delete('SurfaceVerticesEvolver.txt');


%-----Define other parameters----------------------------------------------
coeffnumber=5;  %Coefficient number for interpolation
nx=size(h,2);   %Grid elements in x-direction
   

%-----Definition of the constant pressure loss factors K (table 1, eq. 2)--
%-----Solutions from Matlab pdetoolbox and Ayyaswamy 1974 [9]--------------
Ayya_alpha=[1,5,10,20,30,40,50,60];  %different opening angles
Ayya_theta=[0.1,0.5,1,5:5:85,89];    %different contact angles
Ayya_u=zeros(length(Ayya_alpha), length(Ayya_theta));
Ayya_u(1,:)=[0.048578, 0.048590, 0.048605, 0.048720, 0.048859, 0.048992, 0.049120, 0.049244, 0.049363, 0.049479, 0.049592, 0.049704, 0.049814, 0.049923, 0.050033, 0.050143, 0.050256, 0.050371, 0.050490, 0.050613, 0.050715];
Ayya_u(2,:)=[1.0156, 1.0169, 1.0184, 1.0306, 1.0454, 1.0598, 1.0736, 1.0871, 1.1002, 1.1131, 1.1257, 1.1382, 1.1505, 1.1629, 1.1754, 1.1880, 1.2008, 1.2140, 1.2276, 1.2480, 0];
Ayya_u(3,:)=[3.2329, 3.2409, 3.2509, 3.3333, 3.4270, 3.5220, 3.6151, 3.7063, 3.7959, 3.8842, 3.9716, 4.0586, 4.1456, 4.2331, 4.3216, 4.4118, 4.5042, 4.5995, 4.6985, 0, 0];
Ayya_u(4,:)=[8.0790, 8.1204, 8.1723, 8.5885, 9.1117, 9.6367, 10.162, 10.687, 11.213, 11.739, 12.269, 12.804, 13.347, 13.900, 14.467, 15.051, 15.657, 0, 0, 0, 0];
Ayya_u(5,:)=[11.092, 11.183, 11.297, 12.223, 13.419, 14.650, 15.910, 17.197, 18.509, 19.848, 21.217, 22.621, 24.067, 25.563, 27.117, 0, 0, 0, 0, 0, 0];
Ayya_u(6,:)=[11.568, 11.707, 11.881, 13.320, 15.238, 17.268, 19.398, 21.622, 23.936, 26.343, 28.847, 31.460, 34.194, 0, 0, 0, 0, 0, 0, 0, 0];
Ayya_u(7,:)=[9.9138, 10.084, 10.299, 12.112, 14.610, 17.347, 20.303, 23.466, 26.831, 30.404, 34.194, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
Ayya_u(8,:)=[6.9893, 7.1688, 7.3929, 9.3135, 12.095, 15.294, 18.872, 22.813, 27.117, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
Ayya_u=Ayya_u./1000;


%-----Calculate the cross-sectional area A(x) and its local contact angle theta(x)--------- 
%Two cases: 
    %theta is variable in x-direction if the contact line is pinned
    %theta is constant in x-direction for the free slip contact line
z_crit=height-((1-sin(alpha+theta))*height*tan(alpha)/cos(alpha+theta)); %critical height, between pinned contact line and free slip
theta_x=ones(1,nx)*pi/2-alpha;      %contact angle in x-direction for each cross-section A
A=ones(1,nx)*height^2*tan(alpha);   %cross-section A(x)
r_W=ones(1,nx)*height/cos(alpha);   %wetted length at the sidewalls - Ayyaswamy 1974 [9]
%Calculate theta(x),A(x),r_W(x) to evaluate eq.2 with table 2
for i=1:nx  %for each cross-section in x-direction
    if h(2,i)<z_crit    %free slip case - Weislogel [6]
        theta_x(i)=theta;
        R=h(2,i)*sin(alpha)/(cos(theta)-sin(alpha));
        r_W(i)=R*cos(alpha+theta)/sin(alpha);
        f=sin(alpha)/(cos(theta)-sin(alpha));
        delta=pi/2-theta-alpha;
        A(i)=h(2,i)^2*f^2*(sin(delta)*(1+1/f)-delta);
    elseif h(2,i)~=height   %pinned case
        R=((height-h(2,i))^2+height^2*tan(alpha)^2)/(2*(height-h(2,i)));
        theta_x(i)=pi/2-alpha-asin(height*tan(alpha)/R);
        A(i)=height^2*tan(alpha)-R^2*asin(height*tan(alpha)/R)+height*tan(alpha)*(R+h(2,i)-height);
    end
end


%-----Calculate the viscous pressure loss (eq. 2) with table 1------------
%The friction factor is calculated in each cross-section, depending on the 
%opening angle and the contact angle theta (eq.2,tab. 2)
%The friction factor K is interpolated from the Ayyaswamy matrix defined above 
    %Local pressure loss
dp_fdx=(Q./A).*(viscosity./(r_W.^2.*interp2(Ayya_theta, Ayya_alpha, Ayya_u, theta_x*180/pi, alpha*180/pi,'linear')));
    %Integrated pressure loss
p_f=zeros(1,nx);
p_f(1)=0;
for i=2:nx
    p_f(i)=p_f(i-1)+dp_fdx(i-1)*(h(1,i)-h(1,i-1));
end


%-----Calculate the convective pressure changes (eq.1)--------------------
p_c=(density*Q^2/2).*(1./(A.^2));


%-----Interpolate the discrete pressure points in x-direction ------------
aa=polyfit(h(1,:),p_f+p_c,coeffnumber);
for i=1:coeffnumber+1
    if isnan(aa(i)) 
        aa(i)=1e8; 
    end
end

%-----Store coefficients in text file for next Evolver iteration----------- 
file=fopen('PressureCoefficients.txt','w');
for i=1:coeffnumber+1
    fprintf(file,'aa[%i]:=%.5e;\n',i,aa(coeffnumber+2-i));
end
fclose(file);

return
